@@ -109,10 +109,19 @@ BOOL sWFd_push_back(sWFd* self, char* str);
 BOOL sWFd_flash(sWFd* self);
 BOOL sWFd_close(sWFd* self);
 
+/*
 typedef struct {
     int mFd;
     string_obj* mBuffer;
 } sRFd;
+*/
+
+typedef struct {
+    int mFd;
+    char* mBuffer;
+    char* mPtr;
+    int mMallocSize;
+} sRFd;
 
 sRFd* sRFd_new(int fd);
 sRFd* sRFd_new2(int fd, char* str);
@@ -123,6 +132,7 @@ int sRFd_read_all_preserve(sRFd* self, string_obj* str);
 int sRFd_read_oneline(sRFd* self, string_obj* str, enum eLineField lf);
 int sRFd_read_oneline_preserve(sRFd* self, string_obj* str, enum eLineField lf);
 int sRFd_read_all_to_buffer(sRFd* self);
+void sRFd_set_buffer(sRFd* self, char* buffer);
 
 void saphire_init(enum eAppType app_type, BOOL job_control, enum eRuntimeScript runtime_script, BOOL run_user_runtimescript);
     // 初期化。saphire_shellなどを実行する前に必ず必要
