@@ -92,6 +92,7 @@ char* gStatmentKindStrs[kCommand+1] = {
     "try",
     "envspace",
     "quote",
+    "time",
 //    "parse",
     "inner_command",
     "command"
@@ -2594,7 +2595,7 @@ void saphire_init(enum eAppType app_type, BOOL job_control, enum eRuntimeScript
 
     gReadBuffers = HASH_NEW(100);
 
-    setenv("SAPHIRE_VERSION", "3.3.5", 1);
+    setenv("SAPHIRE_VERSION", "3.4.0", 1);
     setenv("SAPHIRE_PROMPT", "> ", 1);
     setenv("SAPHIRE_DOCDIR", DOCDIR, 1);
     char* home = getenv("HOME");
@@ -2734,7 +2735,10 @@ sRFd* sRFd_new(int fd)
 {
     sRFd* self = (sRFd*)MALLOC(sizeof(sRFd));
 
-    self->mBuffer = STRING_NEW("");
+    self->mMallocSize = 1024;
+    self->mBuffer = MALLOC(self->mMallocSize);
+    strncpy(self->mBuffer, "", 1024);
+    self->mPtr = self->mBuffer;
     self->mFd = fd;
 
     return self;
@@ -2744,7 +2748,9 @@ sRFd* sRFd_new2(int fd, char* str)
 {
     sRFd* self = (sRFd*)MALLOC(sizeof(sRFd));
 
-    self->mBuffer = STRING_NEW(str);
+    self->mBuffer = STRDUP(str);
+    self->mMallocSize = strlen(str) + 1;
+    self->mPtr = self->mBuffer;
     self->mFd = fd;
 
     return self;
@@ -2768,14 +2774,10 @@ BOOL sRFd_close(sRFd* self)
 int sRFd_read_all(sRFd* self, string_obj* str)
 {
     if(self->mFd == -1) {
-        string_put(str, string_c_str(self->mBuffer));
-        string_put(self->mBuffer, "");
+        string_put(str, self->mPtr);
+        self->mPtr = self->mBuffer + strlen(self->mBuffer);
     }
     else {
-        /// 先に読まれたバッファを書き出しておく
-        string_put(str, string_c_str(self->mBuffer));
-        string_put(self->mBuffer, "");
-
         /// 読み込む
         while(1) {
             char buf[BUFSIZ+1];
@@ -2797,8 +2799,23 @@ int sRFd_read_all(sRFd* self, string_obj* str)
                 return -1;
             }
 
-            string_push_back(str, buf);
+            int len = strlen(self->mBuffer) + strlen(buf) + 1;
+            if(len > self->mMallocSize) {
+                self->mMallocSize = len * 2;
+
+                int n = self->mPtr - self->mBuffer;
+                self->mBuffer = REALLOC(self->mBuffer, self->mMallocSize);
+                self->mPtr = self->mBuffer + n;
+
+                strncat(self->mBuffer, buf, self->mMallocSize);
+            }
+            else {
+                strncat(self->mBuffer, buf, self->mMallocSize);
+            }
         }
+
+        string_put(str, self->mPtr);
+        self->mPtr = self->mBuffer + strlen(self->mBuffer);
     }
 
     return 0;
@@ -2808,7 +2825,7 @@ int sRFd_read_all(sRFd* self, string_obj* str)
 int sRFd_read_all_preserve(sRFd* self, string_obj* str)
 {
     if(self->mFd == -1) {
-        string_put(str, string_c_str(self->mBuffer));
+        string_put(str, self->mPtr);
     }
     else {
         /// 読み込む
@@ -2820,7 +2837,7 @@ int sRFd_read_all_preserve(sRFd* self, string_obj* str)
                 return -1;
             }
             if(r == 0) {
-                if(string_length(self->mBuffer) == 0) {
+                if(self->mPtr == self->mBuffer + strlen(self->mBuffer)) {
                     return 1;
                 }
                 break;
@@ -2832,11 +2849,23 @@ int sRFd_read_all_preserve(sRFd* self, string_obj* str)
                 return -1;
             }
 
-            string_push_back(self->mBuffer, buf);
+            int len = strlen(self->mBuffer) + strlen(buf) + 1;
+            if(len > self->mMallocSize) {
+                self->mMallocSize = len * 2;
+
+                int n = self->mPtr - self->mBuffer;
+                self->mBuffer = REALLOC(self->mBuffer, self->mMallocSize);
+                self->mPtr = self->mBuffer + n;
+
+                strncat(self->mBuffer, buf, self->mMallocSize);
+            }
+            else {
+                strncat(self->mBuffer, buf, self->mMallocSize);
+            }
         }
 
         /// 読み込んだバッファを書き込む
-        string_put(str, string_c_str(self->mBuffer));
+        string_put(str, self->mBuffer);
     }
 
     return 0;
@@ -2857,7 +2886,7 @@ int sRFd_read_all_to_buffer(sRFd* self)
                 return -1;
             }
             if(r == 0) {
-                if(string_length(self->mBuffer) == 0) {
+                if(self->mPtr == self->mBuffer + strlen(self->mBuffer)) {
                     return 1;
                 }
                 break;
@@ -2869,7 +2898,19 @@ int sRFd_read_all_to_buffer(sRFd* self)
                 return -1;
             }
 
-            string_push_back(self->mBuffer, buf);
+            int len = strlen(self->mBuffer) + strlen(buf) + 1;
+            if(len < self->mMallocSize) {
+                strncat(self->mBuffer, buf, self->mMallocSize);
+            }
+            else {
+                self->mMallocSize = len * 2;
+
+                int n = self->mPtr - self->mBuffer;
+                self->mBuffer = REALLOC(self->mBuffer, self->mMallocSize);
+                self->mPtr = self->mBuffer + n;
+
+                strncat(self->mBuffer, buf, self->mMallocSize);
+            }
         }
     }
 
@@ -2880,52 +2921,68 @@ int sRFd_read_all_to_buffer(sRFd* self)
 int sRFd_read_oneline(sRFd* self, string_obj* str, enum eLineField lf)
 {
     if(self->mFd == -1) {
-        string_obj* buffer = self->mBuffer;
         if(gKitutukiSigInt) {
             gKitutukiSigInt = FALSE;
             return -1;
         }
 
         /// バッファーが無い
-        if(strcmp(string_c_str(buffer), "") == 0) {
+        if(*self->mPtr == 0) {
             return 1;
         }
 
         /// 取ったバッファから一行を得て返す ///
-        char *p;
-        int lf_char_size;
         if(lf == kLF) {
-            p = strstr(string_c_str(buffer), "\n"); 
-            lf_char_size = 1;
-        }
-        else if(lf == kCRLF) {
-            p = strstr(string_c_str(buffer), "\r\n"); 
-            lf_char_size = 2;
-        }
-        
-        else if(lf == kCR) {
-            p = strstr(string_c_str(buffer), "\r"); 
-            lf_char_size = 1;
-        }
-        else {
-            p = strstr(string_c_str(buffer), "\a"); 
-            lf_char_size = 1;
-        }
-
-        /// 改行がある
-        if(p) {
-            int n = p - string_c_str(buffer);
-            string_push_back3(str, string_c_str(buffer), n + lf_char_size);
-            string_erase(buffer, 0, n + lf_char_size);
-
+            while(*self->mPtr) {
+                if(*self->mPtr == '\n') {
+                    string_push_back2(str, *self->mPtr++);
+                    break;
+                }
+                else {
+                    string_push_back2(str, *self->mPtr++);
+                }
+            }
 
             return 0;
         }
-        /// 改行が無い
-        else {
-            string_push_back(str, string_c_str(buffer));
+        else if(lf == kCRLF) {
+            while(*self->mPtr) {
+                if(*self->mPtr == '\r' && *(self->mPtr + 1) == '\n') {
+                    string_push_back2(str, *self->mPtr++);
+                    string_push_back2(str, *self->mPtr++);
+                    break;
+                }
+                else {
+                    string_push_back2(str, *self->mPtr++);
+                }
+            }
 
-            string_put(buffer,"");
+            return 0;
+        }
+        
+        else if(lf == kCR) {
+            while(*self->mPtr) {
+                if(*self->mPtr == '\r') {
+                    string_push_back2(str, *self->mPtr++);
+                    break;
+                }
+                else {
+                    string_push_back2(str, *self->mPtr++);
+                }
+            }
+
+            return 0;
+        }
+        else {
+            while(*self->mPtr) {
+                if(*self->mPtr == '\a') {
+                    string_push_back2(str, *self->mPtr++);
+                    break;
+                }
+                else {
+                    string_push_back2(str, *self->mPtr++);
+                }
+            }
 
             return 0;
         }
@@ -2934,24 +2991,60 @@ int sRFd_read_oneline(sRFd* self, string_obj* str, enum eLineField lf)
         while(1) {
             /// 改行が含まれていればbreakする ///
             if(lf == kCR) {
-                if(strstr(string_c_str(self->mBuffer), "\r")) {
-                    break;
+                int flg = 0;
+                char* p = self->mPtr;
+                while(*p) {
+                    if(*p == '\r') {
+                        flg = 1;
+                        break;
+                    }
+                    else {
+                        p++;
+                    }
                 }
+                if(flg) break;
             }
             else if(lf == kLF) {
-                if(strstr(string_c_str(self->mBuffer), "\n")) {
-                    break;
+                int flg = 0;
+                char* p = self->mPtr;
+                while(*p) {
+                    if(*p == '\n') {
+                        flg = 1;
+                        break;
+                    }
+                    else {
+                        p++;
+                    }
                 }
+                if(flg) break;
             }
             else if(lf == kCRLF) {
-                if(strstr(string_c_str(self->mBuffer), "\r\n")) {
-                    break;
+                int flg = 0;
+                char* p = self->mPtr;
+                while(*p) {
+                    if(*p == '\r' && *(p+1) == '\n') {
+                        flg = 1;
+                        break;
+                    }
+                    else {
+                        p++;
+                    }
                 }
+                if(flg) break;
             }
             else {
-                if(strstr(string_c_str(self->mBuffer), "\a")) {
-                    break;
+                int flg = 0;
+                char* p = self->mPtr;
+                while(*p) {
+                    if(*p == '\a') {
+                        flg = 1;
+                        break;
+                    }
+                    else {
+                        p++;
+                    }
                 }
+                if(flg) break;
             }
 
             char buf[BUFSIZ+1];
@@ -2972,48 +3065,79 @@ int sRFd_read_oneline(sRFd* self, string_obj* str, enum eLineField lf)
                 break;
             }
             else {
-                string_push_back(self->mBuffer, buf);
+                int len = strlen(self->mBuffer) + strlen(buf) + 1;
+                if(len < self->mMallocSize) {
+                    strncat(self->mBuffer, buf, self->mMallocSize);
+                }
+                else {
+                    self->mMallocSize = len * 2;
+
+                    int n = self->mPtr - self->mBuffer;
+                    self->mBuffer = REALLOC(self->mBuffer, self->mMallocSize);
+                    self->mPtr = self->mBuffer + n;
+
+                    strncat(self->mBuffer, buf, self->mMallocSize);
+                }
             }
         }
 
         /// バッファーが無い
-        if(strcmp(string_c_str(self->mBuffer), "") == 0) {
+        if(*self->mPtr == 0) {
             return 1;
         }
 
         /// 取ったバッファから一行を得て返す ///
-        char *p;
-        int lf_char_size;
         if(lf == kLF) {
-            p = strstr(string_c_str(self->mBuffer), "\n"); 
-            lf_char_size = 1;
-        }
-        else if(lf == kCRLF) {
-            p = strstr(string_c_str(self->mBuffer), "\r\n"); 
-            lf_char_size = 2;
-        }
-        else if(lf == kCR) {
-            p = strstr(string_c_str(self->mBuffer), "\r"); 
-            lf_char_size = 1;
-        }
-        else {
-            p = strstr(string_c_str(self->mBuffer), "\a"); 
-            lf_char_size = 1;
-        }
-
-        /// 改行がある
-        if(p) {
-            int n = p - string_c_str(self->mBuffer);
-            string_push_back3(str, string_c_str(self->mBuffer), n + lf_char_size);
-            string_erase(self->mBuffer, 0, n + lf_char_size);
+            while(*self->mPtr) {
+                if(*self->mPtr == '\n') {
+                    string_push_back2(str, *self->mPtr++);
+                    break;
+                }
+                else {
+                    string_push_back2(str, *self->mPtr++);
+                }
+            }
 
             return 0;
         }
-        /// 改行が無い
-        else {
-            string_push_back(str, string_c_str(self->mBuffer));
+        else if(lf == kCRLF) {
+            while(*self->mPtr) {
+                if(*self->mPtr == '\r' && *(self->mPtr + 1) == '\n') {
+                    string_push_back2(str, *self->mPtr++);
+                    string_push_back2(str, *self->mPtr++);
+                    break;
+                }
+                else {
+                    string_push_back2(str, *self->mPtr++);
+                }
+            }
 
-            string_put(self->mBuffer,"");
+            return 0;
+        }
+        
+        else if(lf == kCR) {
+            while(*self->mPtr) {
+                if(*self->mPtr == '\r') {
+                    string_push_back2(str, *self->mPtr++);
+                    break;
+                }
+                else {
+                    string_push_back2(str, *self->mPtr++);
+                }
+            }
+
+            return 0;
+        }
+        else {
+            while(*self->mPtr) {
+                if(*self->mPtr == '\a') {
+                    string_push_back2(str, *self->mPtr++);
+                    break;
+                }
+                else {
+                    string_push_back2(str, *self->mPtr++);
+                }
+            }
 
             return 0;
         }
@@ -3024,46 +3148,72 @@ int sRFd_read_oneline(sRFd* self, string_obj* str, enum eLineField lf)
 int sRFd_read_oneline_preserve(sRFd* self, string_obj* str, enum eLineField lf)
 {
     if(self->mFd == -1) {
-        string_obj* buffer = self->mBuffer;
         if(gKitutukiSigInt) {
             gKitutukiSigInt = FALSE;
             return -1;
         }
 
         /// バッファーが無い
-        if(strcmp(string_c_str(buffer), "") == 0) {
+        if(*self->mPtr == 0) {
             return 1;
         }
 
         /// 取ったバッファから一行を得て返す ///
-        char *p;
-        int lf_char_size;
         if(lf == kLF) {
-            p = strstr(string_c_str(buffer), "\n"); 
-            lf_char_size = 1;
+            char* p = self->mPtr;
+            while(*p) {
+                if(*p == '\n') {
+                    string_push_back2(str, *p++);
+                    break;
+                }
+                else {
+                    string_push_back2(str, *p++);
+                }
+            }
+
+            return 0;
         }
         else if(lf == kCRLF) {
-            p = strstr(string_c_str(buffer), "\r\n"); 
-            lf_char_size = 2;
+            char* p = self->mPtr;
+            while(*p) {
+                if(*p == '\r' && *(p + 1) == '\n') {
+                    string_push_back2(str, *p++);
+                    string_push_back2(str, *p++);
+                    break;
+                }
+                else {
+                    string_push_back2(str, *p++);
+                }
+            }
+
+            return 0;
         }
+        
         else if(lf == kCR) {
-            p = strstr(string_c_str(buffer), "\r"); 
-            lf_char_size = 1;
-        }
-        else {
-            p = strstr(string_c_str(buffer), "\a"); 
-            lf_char_size = 1;
-        }
+            char* p = self->mPtr;
+            while(*p) {
+                if(*p == '\r') {
+                    string_push_back2(str, *p++);
+                    break;
+                }
+                else {
+                    string_push_back2(str, *p++);
+                }
+            }
 
-        /// 改行がある
-        if(p) {
-            int n = p - string_c_str(buffer);
-            string_push_back3(str, string_c_str(buffer), n + lf_char_size);
             return 0;
         }
-        /// 改行が無い
         else {
-            string_push_back(str, string_c_str(buffer));
+            char* p = self->mPtr;
+            while(*p) {
+                if(*p == '\a') {
+                    string_push_back2(str, *p++);
+                    break;
+                }
+                else {
+                    string_push_back2(str, *p++);
+                }
+            }
 
             return 0;
         }
@@ -3072,24 +3222,60 @@ int sRFd_read_oneline_preserve(sRFd* self, string_obj* str, enum eLineField lf)
         while(1) {
             /// 改行が含まれていればbreakする ///
             if(lf == kCR) {
-                if(strstr(string_c_str(self->mBuffer), "\r")) {
-                    break;
+                int flg = 0;
+                char* p = self->mPtr;
+                while(*p) {
+                    if(*p == '\r') {
+                        flg = 1;
+                        break;
+                    }
+                    else {
+                        p++;
+                    }
                 }
+                if(flg) break;
             }
             else if(lf == kLF) {
-                if(strstr(string_c_str(self->mBuffer), "\n")) {
-                    break;
+                int flg = 0;
+                char* p = self->mPtr;
+                while(*p) {
+                    if(*p == '\n') {
+                        flg = 1;
+                        break;
+                    }
+                    else {
+                        p++;
+                    }
                 }
+                if(flg) break;
             }
             else if(lf == kCRLF) {
-                if(strstr(string_c_str(self->mBuffer), "\r\n")) {
-                    break;
+                int flg = 0;
+                char* p = self->mPtr;
+                while(*p) {
+                    if(*p == '\r' && *(p+1) == '\n') {
+                        flg = 1;
+                        break;
+                    }
+                    else {
+                        p++;
+                    }
                 }
+                if(flg) break;
             }
             else {
-                if(strstr(string_c_str(self->mBuffer), "\a")) {
-                    break;
+                int flg = 0;
+                char* p = self->mPtr;
+                while(*p) {
+                    if(*p == '\a') {
+                        flg = 1;
+                        break;
+                    }
+                    else {
+                        p++;
+                    }
                 }
+                if(flg) break;
             }
 
             char buf[BUFSIZ+1];
@@ -3110,45 +3296,83 @@ int sRFd_read_oneline_preserve(sRFd* self, string_obj* str, enum eLineField lf)
                 break;
             }
             else {
-                string_push_back(self->mBuffer, buf);
+                int len = strlen(self->mBuffer) + strlen(buf) + 1;
+                if(len < self->mMallocSize) {
+                    strncat(self->mBuffer, buf, self->mMallocSize);
+                }
+                else {
+                    self->mMallocSize = len * 2;
+
+                    int n = self->mPtr - self->mBuffer;
+                    self->mBuffer = REALLOC(self->mBuffer, self->mMallocSize);
+                    self->mPtr = self->mBuffer + n;
+
+                    strncat(self->mBuffer, buf, self->mMallocSize);
+                }
             }
         }
 
         /// バッファーが無い
-        if(strcmp(string_c_str(self->mBuffer), "") == 0) {
+        if(*self->mPtr == 0) {
             return 1;
         }
 
         /// 取ったバッファから一行を得て返す ///
-        char *p;
-        int lf_char_size;
         if(lf == kLF) {
-            p = strstr(string_c_str(self->mBuffer), "\n"); 
-            lf_char_size = 1;
+            char* p = self->mPtr;
+            while(*p) {
+                if(*p == '\n') {
+                    string_push_back2(str, *p++);
+                    break;
+                }
+                else {
+                    string_push_back2(str, *p++);
+                }
+            }
+
+            return 0;
         }
         else if(lf == kCRLF) {
-            p = strstr(string_c_str(self->mBuffer), "\r\n"); 
-            lf_char_size = 2;
+            char* p = self->mPtr;
+            while(*p) {
+                if(*p == '\r' && *(p + 1) == '\n') {
+                    string_push_back2(str, *p++);
+                    string_push_back2(str, *p++);
+                    break;
+                }
+                else {
+                    string_push_back2(str, *p++);
+                }
+            }
+
+            return 0;
         }
+        
         else if(lf == kCR) {
-            p = strstr(string_c_str(self->mBuffer), "\r"); 
-            lf_char_size = 1;
-       }
-        else {
-            p = strstr(string_c_str(self->mBuffer), "\a"); 
-            lf_char_size = 1;
-        }
-
-        /// 改行がある
-        if(p) {
-            int n = p - string_c_str(self->mBuffer);
-            string_push_back3(str, string_c_str(self->mBuffer), n + lf_char_size);
+            char* p = self->mPtr;
+            while(*p) {
+                if(*p == '\r') {
+                    string_push_back2(str, *p++);
+                    break;
+                }
+                else {
+                    string_push_back2(str, *p++);
+                }
+            }
 
             return 0;
         }
-        /// 改行が無い
         else {
-            string_push_back(str, string_c_str(self->mBuffer));
+            char* p = self->mPtr;
+            while(*p) {
+                if(*p == '\a') {
+                    string_push_back2(str, *p++);
+                    break;
+                }
+                else {
+                    string_push_back2(str, *p++);
+                }
+            }
 
             return 0;
         }
@@ -3159,62 +3383,88 @@ int sRFd_read_oneline_preserve(sRFd* self, string_obj* str, enum eLineField lf)
 int sRFd_read_oneline_preserve_num(sRFd* self, string_obj* str, int line_num, enum eLineField lf)
 {
     if(self->mFd == -1) {
-        string_obj* buffer = self->mBuffer;
         if(gKitutukiSigInt) {
             gKitutukiSigInt = FALSE;
             return -1;
         }
 
         /// バッファーが無い
-        if(strcmp(string_c_str(buffer), "") == 0) {
+        if(*self->mPtr == 0) {
             return 1;
         }
 
         /// 取ったバッファからn行を得て返す ///
-        char *p;
-        int lf_char_size;
         if(lf == kLF) {
-            int i;
-            p = strstr(string_c_str(buffer), "\n"); 
-            for(i=0; i<line_num-1 && p != NULL; i++) {
-                p = strstr(p+1, "\n"); 
+            int c = 0;
+            char* p = self->mPtr;
+            while(*p) {
+                if(*p == '\n') {
+                    string_push_back2(str, *p++);
+                    c++;
+                    if(c == line_num) {
+                        break;
+                    }
+                }
+                else {
+                    string_push_back2(str, *p++);
+                }
+            }
+
+            return 0;
+        }
+        else if(lf == kCRLF) {
+            char* p = self->mPtr;
+            int c = 0;
+            while(*p) {
+                if(*p == '\r' && *(p + 1) == '\n') {
+                    string_push_back2(str, *p++);
+                    string_push_back2(str, *p++);
+                    c++;
+                    if(c == line_num) {
+                        break;
+                    }
+                }
+                else {
+                    string_push_back2(str, *p++);
+                }
+            }
+
+            return 0;
+        }
+        
+        else if(lf == kCR) {
+            int c = 0;
+            char* p = self->mPtr;
+            while(*p) {
+                if(*p == '\r') {
+                    string_push_back2(str, *p++);
+                    c++;
+                    if(c == line_num) {
+                        break;
+                    }
+                }
+                else {
+                    string_push_back2(str, *p++);
+                }
             }
-            lf_char_size = 1;
-        }
-        else if(lf == kCRLF) {
-            int i;
-            p = strstr(string_c_str(buffer), "\r\n"); 
-            for(i=0; i<line_num-1 && p != NULL; i++) {
-                p = strstr(p+2, "\r\n"); 
-            }
-            lf_char_size = 2;
-        }
-        else if(lf == kCR) {
-            int i;
-            p = strstr(string_c_str(buffer), "\r"); 
-            for(i=0; i<line_num-1 && p != NULL; i++) {
-                p = strstr(p+1, "\r"); 
-            }
-            lf_char_size = 1;
-        }
-        else {
-            int i;
-            p = strstr(string_c_str(buffer), "\a"); 
-            for(i=0; i<line_num-1 && p != NULL; i++) {
-                p = strstr(p+1, "\a"); 
-            }
-            lf_char_size = 1;
-        }
 
-        /// 改行がある
-        if(p) {
-            int n = p - string_c_str(buffer);
-            string_push_back3(str, string_c_str(buffer), n + lf_char_size);
             return 0;
         }
-        /// 改行が無い
         else {
-            string_push_back(str, string_c_str(buffer));
+            char* p = self->mPtr;
+            int c = 0;
+            while(*p) {
+                if(*p == '\a') {
+                    string_push_back2(str, *p++);
+                    c++;
+                    if(c == line_num) {
+                        break;
+                    }
+                }
+                else {
+                    string_push_back2(str, *p++);
+                }
+            }
 
             return 0;
         }
@@ -3223,49 +3473,76 @@ int sRFd_read_oneline_preserve_num(sRFd* self, string_obj* str, int line_num, en
         while(1) {
             /// 改行がn個含まれていればbreakする ///
             if(lf == kCR) {
-                char* p = strstr(string_c_str(self->mBuffer), "\r");
-                int n = 0;
-                while(p != NULL && n<line_num) {
-                    p = strstr(p+1, "\r");
-                    n++;
-                }
-                if(n == line_num) {
-                    break;
+                int flg = 0;
+                int c = 0;
+                char* p = self->mPtr;
+                while(*p) {
+                    if(*p == '\r') {
+                        c++;
+                        if(c == line_num) {
+                            flg = 1;
+                            break;
+                        }
+                    }
+                    else {
+                        p++;
+                    }
                 }
+                if(flg) break;
             }
             else if(lf == kLF) {
-                char* p = strstr(string_c_str(self->mBuffer), "\n");
-                int n = 0;
-                while(p != NULL && n<line_num) {
-                    p = strstr(p+1, "\n");
-                    n++;
-                }
-                if(n == line_num) {
-                    break;
+                int c;
+                int flg = 0;
+                char* p = self->mPtr;
+                while(*p) {
+                    if(*p == '\n') {
+                        c++;
+                        if(c == line_num) {
+                            flg = 1;
+                            break;
+                        }
+                    }
+                    else {
+                        p++;
+                    }
                 }
+                if(flg) break;
             }
             else if(lf == kCRLF) {
-                char* p = strstr(string_c_str(self->mBuffer), "\r\n");
-                int n = 0;
-                while(p != NULL && n<line_num) 
-                {
-                    p = strstr(p+2, "\r\n");
-                    n++;
-                }
-                if(n == line_num) {
-                    break;
+                int c;
+                int flg = 0;
+                char* p = self->mPtr;
+                while(*p) {
+                    if(*p == '\r' && *(p+1) == '\n') {
+                        c++;
+                        if(c == line_num) {
+                            flg = 1;
+                            break;
+                        }
+                    }
+                    else {
+                        p++;
+                    }
                 }
+                if(flg) break;
             }
             else {
-                char* p = strstr(string_c_str(self->mBuffer), "\a");
-                int n = 0;
-                while(p != NULL && n<line_num) {
-                    p = strstr(p+1, "\a");
-                    n++;
-                }
-                if(n == line_num) {
-                    break;
+                int flg = 0;
+                char* p = self->mPtr;
+                int c = 0;
+                while(*p) {
+                    if(*p == '\a') {
+                        c++;
+                        if(c == line_num) {
+                            flg = 1;
+                            break;
+                        }
+                    }
+                    else {
+                        p++;
+                    }
                 }
+                if(flg) break;
             }
 
             char buf[BUFSIZ+1];
@@ -3286,67 +3563,112 @@ int sRFd_read_oneline_preserve_num(sRFd* self, string_obj* str, int line_num, en
                 break;
             }
             else {
-                string_push_back(self->mBuffer, buf);
+                int len = strlen(self->mBuffer) + strlen(buf) + 1;
+                if(len < self->mMallocSize) {
+                    strncat(self->mBuffer, buf, self->mMallocSize);
+                }
+                else {
+                    self->mMallocSize = len * 2;
+
+                    int n = self->mPtr - self->mBuffer;
+                    self->mBuffer = REALLOC(self->mBuffer, self->mMallocSize);
+                    self->mPtr = self->mBuffer + n;
+
+                    strncat(self->mBuffer, buf, self->mMallocSize);
+                }
             }
         }
 
         /// バッファーが無い
-        if(strcmp(string_c_str(self->mBuffer), "") == 0) {
+        if(*self->mPtr == 0) {
             return 1;
         }
 
         /// 取ったバッファからn行を得て返す ///
-        char *p;
-        int lf_char_size;
         if(lf == kLF) {
-            int i;
-            p = strstr(string_c_str(self->mBuffer), "\n"); 
-            for(i=0; i<line_num-1 && p != NULL; i++) {
-                p = strstr(p+1, "\n"); 
+            int c = 0;
+            char* p = self->mPtr;
+            while(*p) {
+                if(*p == '\n') {
+                    string_push_back2(str, *p++);
+                    c++;
+                    if(c == line_num) {
+                        break;
+                    }
+                }
+                else {
+                    string_push_back2(str, *p++);
+                }
+            }
+
+            return 0;
+        }
+        else if(lf == kCRLF) {
+            char* p = self->mPtr;
+            int c = 0;
+            while(*p) {
+                if(*p == '\r' && *(p + 1) == '\n') {
+                    string_push_back2(str, *p++);
+                    string_push_back2(str, *p++);
+                    c++;
+                    if(c == line_num) {
+                        break;
+                    }
+                }
+                else {
+                    string_push_back2(str, *p++);
+                }
+            }
+
+            return 0;
+        }
+        
+        else if(lf == kCR) {
+            int c = 0;
+            char* p = self->mPtr;
+            while(*p) {
+                if(*p == '\r') {
+                    string_push_back2(str, *p++);
+                    c++;
+                    if(c == line_num) {
+                        break;
+                    }
+                }
+                else {
+                    string_push_back2(str, *p++);
+                }
             }
-            lf_char_size = 1;
-        }
-        else if(lf == kCRLF) {
-            int i;
-            p = strstr(string_c_str(self->mBuffer), "\r\n"); 
-            for(i=0; i<line_num-1 && p != NULL; i++) {
-                p = strstr(p+2, "\r\n"); 
-            }
-            lf_char_size = 2;
-        }
-        else if(lf == kCR) {
-            int i;
-            p = strstr(string_c_str(self->mBuffer), "\r"); 
-            for(i=0; i<line_num-1 && p != NULL; i++) {
-                p = strstr(p+1, "\r"); 
-            }
-            lf_char_size = 1;
-        }
-        else {
-            int i;
-            p = strstr(string_c_str(self->mBuffer), "\a"); 
-            for(i=0; i<line_num-1 && p != NULL; i++) {
-                p = strstr(p+1, "\a"); 
-            }
-            lf_char_size = 1;
-        }
-
-        /// 改行がある
-        if(p) {
-            int n = p - string_c_str(self->mBuffer);
-            string_push_back3(str, string_c_str(self->mBuffer), n + lf_char_size);
 
             return 0;
         }
-        /// 改行が無い
         else {
-            string_push_back(str, string_c_str(self->mBuffer));
+            char* p = self->mPtr;
+            int c = 0;
+            while(*p) {
+                if(*p == '\a') {
+                    string_push_back2(str, *p++);
+                    c++;
+                    if(c == line_num) {
+                        break;
+                    }
+                }
+                else {
+                    string_push_back2(str, *p++);
+                }
+            }
 
             return 0;
         }
     }
 }
 
+void sRFd_set_buffer(sRFd* self, char* buffer)
+{
+    self->mBuffer = STRDUP(buffer);
+    self->mMallocSize = strlen(buffer);
+    self->mPtr = self->mBuffer;
+}
+
 
 sWFd* sWFd_new(int fd)
 {
