@@ -339,7 +339,7 @@ static BOOL statment_tree_external_command(sStatment* statment, vector_obj* argv
                 if(mpipefds[1] != -1) {
                     (void)close(nextin2);
                     BOOL epipe = FALSE;
-                    if(!gSigUser && write(mpipefds[1], string_c_str(nextin->mBuffer), string_length(nextin->mBuffer)) < 0)
+                    if(!gSigUser && write(mpipefds[1], nextin->mBuffer, strlen(nextin->mBuffer)) < 0)
                     {
                         if(errno != EPIPE ) {
                             perror("write memory pipe");
@@ -715,10 +715,22 @@ static BOOL statment_tree_make_argv(sCommand* command, vector_obj* argv, vector_
                 vector_add(command->mRedirects, redirect2);
 
                 if(redirect2->mType == kRedirectErrAndOutput) {
-                    redirect = NULL;
+                    if(redirect) {
+                        err_msg("invalid redirect file name", sname, sline);
+                        return FALSE;
+                    }
+                    else {
+                        redirect = NULL;
+                    }
                 }
                 else {
-                    redirect = arg->mBody;
+                    if(redirect) {
+                        err_msg("invalid redirect file name", sname, sline);
+                        return FALSE;
+                    }
+                    else {
+                        redirect = arg->mBody;
+                    }
                 }
                 }
                 break;
@@ -748,21 +760,21 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
 
     if(exist_globalin == 1) {
         nextin = sRFd_new(-1);
-        string_put(nextin->mBuffer, string_c_str(gGlobalPipe));
+        sRFd_set_buffer(nextin, string_c_str(gGlobalPipe));
     }
     else if(exist_globalin == 4) {
         int num = statment->mGlobalPipeInNum;
         nextin = sRFd_new(-1);
-        string_put(nextin->mBuffer, string_c_str(gGlobalPipeNum[num]));
+        sRFd_set_buffer(nextin, string_c_str(gGlobalPipeNum[num]));
     }
     else if(exist_stdinpipe) {
         if(pipein->mFd == -1) {
             nextin = sRFd_new(-1);
-            string_put(nextin->mBuffer, string_c_str(pipein->mBuffer));
+            sRFd_set_buffer(nextin, pipein->mBuffer);
         }
         else {
             sRFd_read_all_to_buffer(pipein);
-            nextin = sRFd_new2(pipein->mFd, string_c_str(pipein->mBuffer));
+            nextin = sRFd_new2(pipein->mFd, pipein->mBuffer);
         }
     }
     else {
@@ -1428,7 +1440,7 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
             }
 
             /// リードバッファが空じゃないなら
-            if(nextin->mFd != -1 && string_c_str(nextin->mBuffer)[0] != 0)
+            if(nextin->mFd != -1 && nextin->mBuffer[0] != 0)
             {
                 int ret = sRFd_read_all_to_buffer(nextin);
 
@@ -1499,7 +1511,7 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
             /// 次の入力先を決める
             if(nextout->mFd == -1) {
                 nextin = sRFd_new(-1);
-                string_put(nextin->mBuffer, nextout->mBuffer);
+                sRFd_set_buffer(nextin, nextout->mBuffer);
                 *nextout->mBuffer = 0;
             }
             else {
